@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
 
 import json
+import math
 import pytest
+import time
 from typing import (
     TYPE_CHECKING,
     Callable,
@@ -37,8 +39,10 @@ from web3.exceptions import (
     BlockNotFound,
     ContractLogicError,
     InvalidAddress,
+    InvalidTransaction,
     NameNotFound,
     TransactionNotFound,
+    TransactionTypeMismatch,
 )
 from web3.types import (  # noqa: F401
     BlockData,
@@ -58,6 +62,17 @@ if TYPE_CHECKING:
     from web3.contract import Contract  # noqa: F401
 
 
+def mine_pending_block(web3: "Web3") -> None:
+    timeout = 10
+
+    web3.geth.miner.start()  # type: ignore
+    start = time.time()
+    while time.time() < start + timeout:
+        if len(web3.eth.get_block('pending')['transactions']) == 0:
+            break
+    web3.geth.miner.stop()  # type: ignore
+
+
 class AsyncEthModuleTest:
     @pytest.mark.asyncio
     async def test_eth_gas_price(self, async_w3: "Web3") -> None:
@@ -70,7 +85,7 @@ class AsyncEthModuleTest:
         assert is_connected is True
 
     @pytest.mark.asyncio
-    async def test_eth_send_transaction(
+    async def test_eth_send_transaction_legacy(
         self, async_w3: "Web3", unlocked_account_dual_type: ChecksumAddress
     ) -> None:
         txn_params: TxParams = {
@@ -89,6 +104,161 @@ class AsyncEthModuleTest:
         assert txn['gas'] == 21000
         assert txn['gasPrice'] == txn_params['gasPrice']
 
+    @pytest.mark.asyncio
+    async def test_eth_send_transaction(
+        self, async_w3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+            'maxFeePerGas': async_w3.toWei(3, 'gwei'),
+            'maxPriorityFeePerGas': async_w3.toWei(1, 'gwei'),
+        }
+        txn_hash = await async_w3.eth.send_transaction(txn_params)  # type: ignore
+        txn = await async_w3.eth.get_transaction(txn_hash)  # type: ignore
+
+        assert is_same_address(txn['from'], cast(ChecksumAddress, txn_params['from']))
+        assert is_same_address(txn['to'], cast(ChecksumAddress, txn_params['to']))
+        assert txn['value'] == 1
+        assert txn['gas'] == 21000
+        assert txn['maxFeePerGas'] == txn_params['maxFeePerGas']
+        assert txn['maxPriorityFeePerGas'] == txn_params['maxPriorityFeePerGas']
+        assert txn['gasPrice'] is None
+
+    @pytest.mark.asyncio
+    async def test_eth_send_transaction_default_fees(
+        self, async_w3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+        }
+        txn_hash = await async_w3.eth.send_transaction(txn_params)  # type: ignore
+        txn = await async_w3.eth.get_transaction(txn_hash)  # type: ignore
+
+        assert is_same_address(txn['from'], cast(ChecksumAddress, txn_params['from']))
+        assert is_same_address(txn['to'], cast(ChecksumAddress, txn_params['to']))
+        assert txn['value'] == 1
+        assert txn['gas'] == 21000
+        assert txn['maxPriorityFeePerGas'] == 1 * 10**9
+        assert txn['maxFeePerGas'] >= 1 * 10**9
+        assert txn['gasPrice'] is None
+
+    @pytest.mark.asyncio
+    async def test_eth_send_transaction_hex_fees(
+        self, async_w3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+            'maxFeePerGas': hex(250 * 10**9),
+            'maxPriorityFeePerGas': hex(2 * 10**9),
+        }
+        txn_hash = await async_w3.eth.send_transaction(txn_params)  # type: ignore
+        txn = await async_w3.eth.get_transaction(txn_hash)  # type: ignore
+
+        assert is_same_address(txn['from'], cast(ChecksumAddress, txn_params['from']))
+        assert is_same_address(txn['to'], cast(ChecksumAddress, txn_params['to']))
+        assert txn['value'] == 1
+        assert txn['gas'] == 21000
+        assert txn['maxFeePerGas'] == 250 * 10**9
+        assert txn['maxPriorityFeePerGas'] == 2 * 10**9
+
+    @pytest.mark.asyncio
+    async def test_eth_send_transaction_no_gas(
+        self, async_w3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'maxFeePerGas': Wei(250 * 10**9),
+            'maxPriorityFeePerGas': Wei(2 * 10**9),
+        }
+        txn_hash = await async_w3.eth.send_transaction(txn_params)  # type: ignore
+        txn = await async_w3.eth.get_transaction(txn_hash)  # type: ignore
+
+        assert is_same_address(txn['from'], cast(ChecksumAddress, txn_params['from']))
+        assert is_same_address(txn['to'], cast(ChecksumAddress, txn_params['to']))
+        assert txn['value'] == 1
+        assert txn['gas'] == 121000  # 21000 + buffer
+
+    @pytest.mark.asyncio
+    async def test_eth_send_transaction_with_gas_price(
+        self, async_w3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+            'gasPrice': Wei(1),
+            'maxFeePerGas': Wei(250 * 10**9),
+            'maxPriorityFeePerGas': Wei(2 * 10**9),
+        }
+        with pytest.raises(TransactionTypeMismatch):
+            await async_w3.eth.send_transaction(txn_params)  # type: ignore
+
+    @pytest.mark.asyncio
+    async def test_eth_send_transaction_no_priority_fee(
+        self, async_w3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+            'maxFeePerGas': Wei(250 * 10**9),
+        }
+        with pytest.raises(InvalidTransaction, match='maxPriorityFeePerGas must be defined'):
+            await async_w3.eth.send_transaction(txn_params)  # type: ignore
+
+    @pytest.mark.asyncio
+    async def test_eth_send_transaction_no_max_fee(
+        self, async_w3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        maxPriorityFeePerGas = async_w3.toWei(2, 'gwei')
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+            'maxPriorityFeePerGas': maxPriorityFeePerGas,
+        }
+        txn_hash = await async_w3.eth.send_transaction(txn_params)  # type: ignore
+        txn = await async_w3.eth.get_transaction(txn_hash)  # type: ignore
+
+        assert is_same_address(txn['from'], cast(ChecksumAddress, txn_params['from']))
+        assert is_same_address(txn['to'], cast(ChecksumAddress, txn_params['to']))
+        assert txn['value'] == 1
+        assert txn['gas'] == 21000
+
+        block = await async_w3.eth.get_block('latest')  # type: ignore
+        assert txn['maxFeePerGas'] == maxPriorityFeePerGas + 2 * block['baseFeePerGas']
+
+    @pytest.mark.asyncio
+    async def test_eth_send_transaction_max_fee_less_than_tip(
+        self, async_w3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+            'maxFeePerGas': Wei(1 * 10**9),
+            'maxPriorityFeePerGas': Wei(2 * 10**9),
+        }
+        with pytest.raises(
+            InvalidTransaction, match="maxFeePerGas must be >= maxPriorityFeePerGas"
+        ):
+            await async_w3.eth.send_transaction(txn_params)  # type: ignore
+
     @pytest.mark.asyncio
     async def test_gas_price_strategy_middleware(
         self, async_w3: "Web3", unlocked_account_dual_type: ChecksumAddress
@@ -99,16 +269,18 @@ class AsyncEthModuleTest:
             'value': Wei(1),
             'gas': Wei(21000),
         }
+        two_gwei_in_wei = async_w3.toWei(2, 'gwei')
 
-        def higher_gas_price_strategy(web3: "Web3", txn: TxParams) -> Wei:
-            return Wei(20)
+        def gas_price_strategy(web3: "Web3", txn: TxParams) -> Wei:
+            return two_gwei_in_wei
 
-        async_w3.eth.set_gas_price_strategy(higher_gas_price_strategy)
+        async_w3.eth.set_gas_price_strategy(gas_price_strategy)
 
         txn_hash = await async_w3.eth.send_transaction(txn_params)  # type: ignore
         txn = await async_w3.eth.get_transaction(txn_hash)  # type: ignore
 
-        assert txn['gasPrice'] == 20
+        assert txn['gasPrice'] == two_gwei_in_wei
+        async_w3.eth.set_gas_price_strategy(None)  # reset strategy
 
     @pytest.mark.asyncio
     async def test_eth_estimate_gas(
@@ -130,9 +302,7 @@ class AsyncEthModuleTest:
         assert block['hash'] == empty_block['hash']
 
     @pytest.mark.asyncio
-    async def test_eth_getBlockByHash_not_found(
-        self, async_w3: "Web3", empty_block: BlockData
-    ) -> None:
+    async def test_eth_getBlockByHash_not_found(self, async_w3: "Web3") -> None:
         with pytest.raises(BlockNotFound):
             await async_w3.eth.get_block(UNKNOWN_HASH)  # type: ignore
 
@@ -777,7 +947,8 @@ class EthModuleTest:
             'to': unlocked_account,
             'value': Wei(1),
             'gas': Wei(21000),
-            'gasPrice': web3.eth.gas_price,
+            'maxFeePerGas': web3.toWei(2, 'gwei'),
+            'maxPriorityFeePerGas': web3.toWei(1, 'gwei'),
         }
 
         with pytest.raises(InvalidAddress):
@@ -788,7 +959,7 @@ class EthModuleTest:
             invalid_params = cast(TxParams, dict(txn_params, **{'to': non_checksum_addr}))
             web3.eth.send_transaction(invalid_params)
 
-    def test_eth_send_transaction(
+    def test_eth_send_transaction_legacy(
         self, web3: "Web3", unlocked_account_dual_type: ChecksumAddress
     ) -> None:
         txn_params: TxParams = {
@@ -807,6 +978,28 @@ class EthModuleTest:
         assert txn['gas'] == 21000
         assert txn['gasPrice'] == txn_params['gasPrice']
 
+    def test_eth_send_transaction(
+        self, web3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+            'maxFeePerGas': web3.toWei(3, 'gwei'),
+            'maxPriorityFeePerGas': web3.toWei(1, 'gwei'),
+        }
+        txn_hash = web3.eth.send_transaction(txn_params)
+        txn = web3.eth.get_transaction(txn_hash)
+
+        assert is_same_address(txn['from'], cast(ChecksumAddress, txn_params['from']))
+        assert is_same_address(txn['to'], cast(ChecksumAddress, txn_params['to']))
+        assert txn['value'] == 1
+        assert txn['gas'] == 21000
+        assert txn['maxFeePerGas'] == txn_params['maxFeePerGas']
+        assert txn['maxPriorityFeePerGas'] == txn_params['maxPriorityFeePerGas']
+        assert txn['gasPrice'] is None
+
     def test_eth_sendTransaction_deprecated(
         self, web3: "Web3", unlocked_account_dual_type: ChecksumAddress
     ) -> None:
@@ -815,7 +1008,8 @@ class EthModuleTest:
             'to': unlocked_account_dual_type,
             'value': Wei(1),
             'gas': Wei(21000),
-            'gasPrice': web3.eth.gas_price,
+            'maxFeePerGas': web3.toWei(3, 'gwei'),
+            'maxPriorityFeePerGas': web3.toWei(1, 'gwei'),
         }
         with pytest.warns(DeprecationWarning,
                           match="sendTransaction is deprecated in favor of send_transaction"):
@@ -826,18 +1020,23 @@ class EthModuleTest:
         assert is_same_address(txn['to'], cast(ChecksumAddress, txn_params['to']))
         assert txn['value'] == 1
         assert txn['gas'] == 21000
-        assert txn['gasPrice'] == txn_params['gasPrice']
+        assert txn['maxFeePerGas'] == txn_params['maxFeePerGas']
+        assert txn['maxPriorityFeePerGas'] == txn_params['maxPriorityFeePerGas']
+        assert txn['gasPrice'] is None
 
     def test_eth_send_transaction_with_nonce(
         self, web3: "Web3", unlocked_account: ChecksumAddress
     ) -> None:
+        mine_pending_block(web3)  # gives an accurate transaction count after mining
+
         txn_params: TxParams = {
             'from': unlocked_account,
             'to': unlocked_account,
             'value': Wei(1),
             'gas': Wei(21000),
-            # Increased gas price to ensure transaction hash different from other tests
-            'gasPrice': Wei(web3.eth.gas_price * 3),
+            # unique maxFeePerGas to ensure transaction hash different from other tests
+            'maxFeePerGas': web3.toWei(4.321, 'gwei'),
+            'maxPriorityFeePerGas': web3.toWei(1, 'gwei'),
             'nonce': web3.eth.get_transaction_count(unlocked_account),
         }
         txn_hash = web3.eth.send_transaction(txn_params)
@@ -847,10 +1046,137 @@ class EthModuleTest:
         assert is_same_address(txn['to'], cast(ChecksumAddress, txn_params['to']))
         assert txn['value'] == 1
         assert txn['gas'] == 21000
-        assert txn['gasPrice'] == txn_params['gasPrice']
+        assert txn['maxFeePerGas'] == txn_params['maxFeePerGas']
+        assert txn['maxPriorityFeePerGas'] == txn_params['maxPriorityFeePerGas']
         assert txn['nonce'] == txn_params['nonce']
+        assert txn['gasPrice'] is None
 
-    def test_eth_replace_transaction(
+    def test_eth_send_transaction_default_fees(
+        self, web3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+        }
+        txn_hash = web3.eth.send_transaction(txn_params)
+        txn = web3.eth.get_transaction(txn_hash)
+
+        assert is_same_address(txn['from'], cast(ChecksumAddress, txn_params['from']))
+        assert is_same_address(txn['to'], cast(ChecksumAddress, txn_params['to']))
+        assert txn['value'] == 1
+        assert txn['gas'] == 21000
+        assert txn['maxPriorityFeePerGas'] == 1 * 10**9
+        assert txn['maxFeePerGas'] >= 1 * 10**9
+        assert txn['gasPrice'] is None
+
+    def test_eth_send_transaction_hex_fees(
+        self, web3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+            'maxFeePerGas': hex(250 * 10**9),
+            'maxPriorityFeePerGas': hex(2 * 10**9),
+        }
+        txn_hash = web3.eth.send_transaction(txn_params)
+        txn = web3.eth.get_transaction(txn_hash)
+
+        assert is_same_address(txn['from'], cast(ChecksumAddress, txn_params['from']))
+        assert is_same_address(txn['to'], cast(ChecksumAddress, txn_params['to']))
+        assert txn['value'] == 1
+        assert txn['gas'] == 21000
+        assert txn['maxFeePerGas'] == 250 * 10**9
+        assert txn['maxPriorityFeePerGas'] == 2 * 10**9
+
+    def test_eth_send_transaction_no_gas(
+        self, web3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'maxFeePerGas': Wei(250 * 10**9),
+            'maxPriorityFeePerGas': Wei(2 * 10**9),
+        }
+        txn_hash = web3.eth.send_transaction(txn_params)
+        txn = web3.eth.get_transaction(txn_hash)
+
+        assert is_same_address(txn['from'], cast(ChecksumAddress, txn_params['from']))
+        assert is_same_address(txn['to'], cast(ChecksumAddress, txn_params['to']))
+        assert txn['value'] == 1
+        assert txn['gas'] == 121000  # 21000 + buffer
+
+    def test_eth_send_transaction_with_gas_price(
+        self, web3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+            'gasPrice': Wei(1),
+            'maxFeePerGas': Wei(250 * 10**9),
+            'maxPriorityFeePerGas': Wei(2 * 10**9),
+        }
+        with pytest.raises(TransactionTypeMismatch):
+            web3.eth.send_transaction(txn_params)
+
+    def test_eth_send_transaction_no_priority_fee(
+        self, web3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+            'maxFeePerGas': Wei(250 * 10**9),
+        }
+        with pytest.raises(InvalidTransaction, match='maxPriorityFeePerGas must be defined'):
+            web3.eth.send_transaction(txn_params)
+
+    def test_eth_send_transaction_no_max_fee(
+        self, web3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        maxPriorityFeePerGas = web3.toWei(2, 'gwei')
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+            'maxPriorityFeePerGas': maxPriorityFeePerGas,
+        }
+        txn_hash = web3.eth.send_transaction(txn_params)
+        txn = web3.eth.get_transaction(txn_hash)
+
+        assert is_same_address(txn['from'], cast(ChecksumAddress, txn_params['from']))
+        assert is_same_address(txn['to'], cast(ChecksumAddress, txn_params['to']))
+        assert txn['value'] == 1
+        assert txn['gas'] == 21000
+
+        block = web3.eth.get_block('latest')
+        assert txn['maxFeePerGas'] == maxPriorityFeePerGas + 2 * block['baseFeePerGas']
+
+    def test_eth_send_transaction_max_fee_less_than_tip(
+        self, web3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+            'maxFeePerGas': Wei(1 * 10**9),
+            'maxPriorityFeePerGas': Wei(2 * 10**9),
+        }
+        with pytest.raises(
+            InvalidTransaction, match="maxFeePerGas must be >= maxPriorityFeePerGas"
+        ):
+            web3.eth.send_transaction(txn_params)
+
+    def test_eth_replace_transaction_legacy(
         self, web3: "Web3", unlocked_account_dual_type: ChecksumAddress
     ) -> None:
         txn_params: TxParams = {
@@ -872,7 +1198,36 @@ class EthModuleTest:
         assert replace_txn['gas'] == 21000
         assert replace_txn['gasPrice'] == txn_params['gasPrice']
 
-    def test_eth_replaceTransaction_deprecated(
+    def test_eth_replace_transaction(
+        self, web3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        two_gwei_in_wei = web3.toWei(2, 'gwei')
+        three_gwei_in_wei = web3.toWei(3, 'gwei')
+
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+            'maxFeePerGas': two_gwei_in_wei,
+            'maxPriorityFeePerGas': web3.toWei(1, 'gwei'),
+        }
+        txn_hash = web3.eth.send_transaction(txn_params)
+
+        txn_params['maxFeePerGas'] = three_gwei_in_wei
+        txn_params['maxPriorityFeePerGas'] = two_gwei_in_wei
+
+        replace_txn_hash = web3.eth.replace_transaction(txn_hash, txn_params)
+        replace_txn = web3.eth.get_transaction(replace_txn_hash)
+
+        assert is_same_address(replace_txn['from'], cast(ChecksumAddress, txn_params['from']))
+        assert is_same_address(replace_txn['to'], cast(ChecksumAddress, txn_params['to']))
+        assert replace_txn['value'] == 1
+        assert replace_txn['gas'] == 21000
+        assert replace_txn['maxFeePerGas'] == three_gwei_in_wei
+        assert replace_txn['maxPriorityFeePerGas'] == two_gwei_in_wei
+
+    def test_eth_replace_transaction_underpriced(
         self, web3: "Web3", unlocked_account_dual_type: ChecksumAddress
     ) -> None:
         txn_params: TxParams = {
@@ -880,11 +1235,36 @@ class EthModuleTest:
             'to': unlocked_account_dual_type,
             'value': Wei(1),
             'gas': Wei(21000),
-            'gasPrice': web3.eth.gas_price,
+            'maxFeePerGas': web3.toWei(3, 'gwei'),
+            'maxPriorityFeePerGas': web3.toWei(2, 'gwei'),
         }
         txn_hash = web3.eth.send_transaction(txn_params)
 
-        txn_params['gasPrice'] = Wei(web3.eth.gas_price * 2)
+        one_gwei_in_wei = web3.toWei(1, 'gwei')
+        txn_params['maxFeePerGas'] = one_gwei_in_wei
+        txn_params['maxPriorityFeePerGas'] = one_gwei_in_wei
+
+        with pytest.raises(ValueError, match="replacement transaction underpriced"):
+            web3.eth.replace_transaction(txn_hash, txn_params)
+
+    def test_eth_replaceTransaction_deprecated(
+        self, web3: "Web3", unlocked_account_dual_type: ChecksumAddress
+    ) -> None:
+        two_gwei_in_wei = web3.toWei(2, 'gwei')
+        three_gwei_in_wei = web3.toWei(3, 'gwei')
+
+        txn_params: TxParams = {
+            'from': unlocked_account_dual_type,
+            'to': unlocked_account_dual_type,
+            'value': Wei(1),
+            'gas': Wei(21000),
+            'maxFeePerGas': two_gwei_in_wei,
+            'maxPriorityFeePerGas': web3.toWei(1, 'gwei'),
+        }
+        txn_hash = web3.eth.send_transaction(txn_params)
+
+        txn_params['maxFeePerGas'] = three_gwei_in_wei
+        txn_params['maxPriorityFeePerGas'] = two_gwei_in_wei
         with pytest.warns(
             DeprecationWarning,
             match="replaceTransaction is deprecated in favor of replace_transaction"
@@ -896,7 +1276,8 @@ class EthModuleTest:
         assert is_same_address(replace_txn['to'], cast(ChecksumAddress, txn_params['to']))
         assert replace_txn['value'] == 1
         assert replace_txn['gas'] == 21000
-        assert replace_txn['gasPrice'] == txn_params['gasPrice']
+        assert replace_txn['maxFeePerGas'] == three_gwei_in_wei
+        assert replace_txn['maxPriorityFeePerGas'] == two_gwei_in_wei
 
     def test_eth_replace_transaction_non_existing_transaction(
         self, web3: "Web3", unlocked_account_dual_type: ChecksumAddress
@@ -906,7 +1287,8 @@ class EthModuleTest:
             'to': unlocked_account_dual_type,
             'value': Wei(1),
             'gas': Wei(21000),
-            'gasPrice': web3.eth.gas_price,
+            'maxFeePerGas': web3.toWei(3, 'gwei'),
+            'maxPriorityFeePerGas': web3.toWei(1, 'gwei'),
         }
         with pytest.raises(TransactionNotFound):
             web3.eth.replace_transaction(
@@ -922,12 +1304,18 @@ class EthModuleTest:
             'to': unlocked_account_dual_type,
             'value': Wei(1),
             'gas': Wei(21000),
-            'gasPrice': web3.eth.gas_price,
+            'maxFeePerGas': web3.toWei(2, 'gwei'),
+            'maxPriorityFeePerGas': web3.toWei(1, 'gwei'),
         }
         txn_hash = web3.eth.send_transaction(txn_params)
-        web3.eth.wait_for_transaction_receipt(txn_hash)
+        try:
+            web3.geth.miner.start()  # type: ignore
+            web3.eth.wait_for_transaction_receipt(txn_hash, timeout=10)
+        finally:
+            web3.geth.miner.stop()  # type: ignore
 
-        txn_params['gasPrice'] = Wei(web3.eth.gas_price * 2)
+        txn_params['maxFeePerGas'] = web3.toWei(3, 'gwei')
+        txn_params['maxPriorityFeePerGas'] = web3.toWei(2, 'gwei')
         with pytest.raises(ValueError, match="Supplied transaction with hash"):
             web3.eth.replace_transaction(txn_hash, txn_params)
 
@@ -939,12 +1327,14 @@ class EthModuleTest:
             'to': unlocked_account,
             'value': Wei(1),
             'gas': Wei(21000),
-            'gasPrice': web3.eth.gas_price,
+            'maxFeePerGas': web3.toWei(2, 'gwei'),
+            'maxPriorityFeePerGas': web3.toWei(1, 'gwei'),
         }
         txn_hash = web3.eth.send_transaction(txn_params)
         txn = web3.eth.get_transaction(txn_hash)
 
-        txn_params['gasPrice'] = Wei(web3.eth.gas_price * 2)
+        txn_params['maxFeePerGas'] = web3.toWei(3, 'gwei')
+        txn_params['maxPriorityFeePerGas'] = web3.toWei(2, 'gwei')
         txn_params['nonce'] = Nonce(txn['nonce'] + 1)
         with pytest.raises(ValueError):
             web3.eth.replace_transaction(txn_hash, txn_params)
@@ -957,23 +1347,25 @@ class EthModuleTest:
             'to': unlocked_account_dual_type,
             'value': Wei(1),
             'gas': Wei(21000),
-            'gasPrice': Wei(10),
+            'gasPrice': web3.toWei(2, 'gwei'),
         }
         txn_hash = web3.eth.send_transaction(txn_params)
 
-        txn_params['gasPrice'] = Wei(9)
+        txn_params['gasPrice'] = web3.toWei(1, 'gwei')
         with pytest.raises(ValueError):
             web3.eth.replace_transaction(txn_hash, txn_params)
 
     def test_eth_replace_transaction_gas_price_defaulting_minimum(
         self, web3: "Web3", unlocked_account: ChecksumAddress
     ) -> None:
+        gas_price = web3.toWei(1, 'gwei')
+
         txn_params: TxParams = {
             'from': unlocked_account,
             'to': unlocked_account,
             'value': Wei(1),
             'gas': Wei(21000),
-            'gasPrice': Wei(10),
+            'gasPrice': gas_price,
         }
         txn_hash = web3.eth.send_transaction(txn_params)
 
@@ -981,7 +1373,7 @@ class EthModuleTest:
         replace_txn_hash = web3.eth.replace_transaction(txn_hash, txn_params)
         replace_txn = web3.eth.get_transaction(replace_txn_hash)
 
-        assert replace_txn['gasPrice'] == 12  # minimum gas price
+        assert replace_txn['gasPrice'] == math.ceil(gas_price * 1.125)  # minimum gas price
 
     def test_eth_replace_transaction_gas_price_defaulting_strategy_higher(
         self, web3: "Web3", unlocked_account: ChecksumAddress
@@ -991,34 +1383,39 @@ class EthModuleTest:
             'to': unlocked_account,
             'value': Wei(1),
             'gas': Wei(21000),
-            'gasPrice': Wei(10),
+            'gasPrice': web3.toWei(1, 'gwei'),
         }
         txn_hash = web3.eth.send_transaction(txn_params)
 
+        two_gwei_in_wei = web3.toWei(2, 'gwei')
+
         def higher_gas_price_strategy(web3: "Web3", txn: TxParams) -> Wei:
-            return Wei(20)
+            return two_gwei_in_wei
 
         web3.eth.set_gas_price_strategy(higher_gas_price_strategy)
 
         txn_params.pop('gasPrice')
         replace_txn_hash = web3.eth.replace_transaction(txn_hash, txn_params)
         replace_txn = web3.eth.get_transaction(replace_txn_hash)
-        assert replace_txn['gasPrice'] == 20  # Strategy provides higher gas price
+        assert replace_txn['gasPrice'] == two_gwei_in_wei  # Strategy provides higher gas price
+        web3.eth.set_gas_price_strategy(None)  # reset strategy
 
     def test_eth_replace_transaction_gas_price_defaulting_strategy_lower(
         self, web3: "Web3", unlocked_account: ChecksumAddress
     ) -> None:
+        gas_price = web3.toWei(2, 'gwei')
+
         txn_params: TxParams = {
             'from': unlocked_account,
             'to': unlocked_account,
             'value': Wei(1),
             'gas': Wei(21000),
-            'gasPrice': Wei(10),
+            'gasPrice': gas_price,
         }
         txn_hash = web3.eth.send_transaction(txn_params)
 
         def lower_gas_price_strategy(web3: "Web3", txn: TxParams) -> Wei:
-            return Wei(5)
+            return web3.toWei(1, 'gwei')
 
         web3.eth.set_gas_price_strategy(lower_gas_price_strategy)
 
@@ -1026,9 +1423,10 @@ class EthModuleTest:
         replace_txn_hash = web3.eth.replace_transaction(txn_hash, txn_params)
         replace_txn = web3.eth.get_transaction(replace_txn_hash)
         # Strategy provides lower gas price - minimum preferred
-        assert replace_txn['gasPrice'] == 12
+        assert replace_txn['gasPrice'] == math.ceil(gas_price * 1.125)
+        web3.eth.set_gas_price_strategy(None)  # reset strategy
 
-    def test_eth_modify_transaction(
+    def test_eth_modify_transaction_legacy(
         self, web3: "Web3", unlocked_account: ChecksumAddress
     ) -> None:
         txn_params: TxParams = {
@@ -1051,6 +1449,35 @@ class EthModuleTest:
         assert modified_txn['gas'] == 21000
         assert modified_txn['gasPrice'] == cast(int, txn_params['gasPrice']) * 2
 
+    def test_eth_modify_transaction(
+        self, web3: "Web3", unlocked_account: ChecksumAddress
+    ) -> None:
+        txn_params: TxParams = {
+            'from': unlocked_account,
+            'to': unlocked_account,
+            'value': Wei(1),
+            'gas': Wei(21000),
+            'maxPriorityFeePerGas': web3.toWei(1, 'gwei'),
+            'maxFeePerGas': web3.toWei(2, 'gwei'),
+        }
+        txn_hash = web3.eth.send_transaction(txn_params)
+
+        modified_txn_hash = web3.eth.modify_transaction(
+            txn_hash,
+            value=2,
+            maxPriorityFeePerGas=(cast(Wei, txn_params['maxPriorityFeePerGas']) * 2),
+            maxFeePerGas=(cast(Wei, txn_params['maxFeePerGas']) * 2),
+        )
+        modified_txn = web3.eth.get_transaction(modified_txn_hash)
+
+        assert is_same_address(modified_txn['from'], cast(ChecksumAddress, txn_params['from']))
+        assert is_same_address(modified_txn['to'], cast(ChecksumAddress, txn_params['to']))
+        assert modified_txn['value'] == 2
+        assert modified_txn['gas'] == 21000
+        assert modified_txn['maxPriorityFeePerGas'] == cast(Wei, txn_params[
+            'maxPriorityFeePerGas']) * 2
+        assert modified_txn['maxFeePerGas'] == cast(Wei, txn_params['maxFeePerGas']) * 2
+
     def test_eth_modifyTransaction_deprecated(
         self, web3: "Web3", unlocked_account: ChecksumAddress
     ) -> None:
@@ -1418,7 +1845,8 @@ class EthModuleTest:
             'to': unlocked_account_dual_type,
             'value': Wei(1),
             'gas': Wei(21000),
-            'gasPrice': web3.eth.gas_price,
+            'maxFeePerGas': web3.toWei(3, 'gwei'),
+            'maxPriorityFeePerGas': web3.toWei(1, 'gwei')
         })
         with pytest.raises(TransactionNotFound):
             web3.eth.get_transaction_receipt(txn_hash)
